extern crate downcast_rs;
extern crate ncurses;

use ncurses::*;

use std::thread::sleep;
use std::time;

mod lib {
    pub mod ai;
    pub mod entity;
    pub mod point;
    pub mod screen;
    pub mod world;
}

use lib::entity::{Ant, Entities, FoodGenerator};
use lib::point::Point;
use lib::screen::init_screen;
use lib::world::{render, simulate, World};

fn main() {
    let mut screen = init_screen();
    let mut world = World::new(&Point(screen.max_x, screen.max_y));
    world.home.insert(screen.center);

    let mut entities = Entities::new();

    let fg = FoodGenerator::new();
    entities.add_entity(&fg);

    for _ in 0..5 {
        let mut a = Ant::new(screen.center.0, screen.center.1);
        a.goal = lib::ai::AIGoal::Seek;
        entities.add_entity(&a);
    }

    let mut t = 0;
    loop {
        // TODO: add way to break out of the loop by hitting a random key
        simulate(&mut entities, &mut world, &mut screen, t);
        render(&entities, &world, &screen);
        sleep(time::Duration::from_millis(100));
        refresh();
        t += 1;
    }
    endwin();
}
